@@ -1498,6 +1498,54 @@ class Backend(Database):
 
         raise NoCustomerMatch("No customer lookup configured for user '{}' or '{}'".format(login, ','.join(matches)))
 
+    # NOTES
+
+    def create_note(self, note):
+        data = {
+            '_id': note.id,
+            'text': note.text,
+            'user': note.user,
+            'attributes': note.attributes,
+            'type': note.note_type,
+            'createTime': note.create_time,
+            'updateTime': note.update_time,
+            'alert': note.alert
+        }
+        if note.customer:
+            data['customer'] = note.customer
+
+        if self.get_db().notes.insert_one(data).inserted_id == note.id:
+            return data
+
+    def get_note(self, id):
+        query = {'_id': id}
+        return self.get_db().notes.find_one(query)
+
+    def get_notes(self, query=None, page=None, page_size=None):
+        query = query or Query()
+        return self.get_db().notes.find(query.where, sort=query.sort).skip((page - 1) * page_size).limit(page_size)
+
+    def get_alert_notes(self, id, page=None, page_size=None):
+        if len(id) == 8:
+            query = {'alert': {'$regex': '^' + id}}
+        else:
+            query = {'alert': id}
+        return self.get_db().notes.find(query).skip((page - 1) * page_size).limit(page_size)
+
+    def get_customer_notes(self, customer, page=None, page_size=None):
+        return self.get_db().notes.find({'customer': customer}).skip((page - 1) * page_size).limit(page_size)
+
+    def update_note(self, id, **kwargs):
+        return self.get_db().notes.find_one_and_update(
+            {'_id': id},
+            update={'$set': kwargs},
+            return_document=ReturnDocument.AFTER
+        )
+
+    def delete_note(self, id):
+        response = self.get_db().notes.delete_one({'_id': id})
+        return True if response.deleted_count == 1 else False
+
     # METRICS
 
     def get_metrics(self, type=None):
@@ -1587,10 +1635,10 @@ class Backend(Database):
                 'event': 1, 'status': 1, 'lastReceiveId': 1, 'timeout': 1,
                 'expireTime': {'$add': ['$lastReceiveTime', {'$multiply': ['$timeout', 1000]}]}}
              },
-            {'$match': {'status': {'$nin': ['expired', 'shelved']}, 'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {
+            {'$match': {'status': {'$nin': ['expired', 'ack', 'shelved']}, 'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {
                 '$ne': 0}}}
         ]
-        expired = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
+        has_expired = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
 
         # get list of alerts to be unshelved
         pipeline = [
@@ -1615,6 +1663,31 @@ class Backend(Database):
             }},
             {'$match': {'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {'$ne': 0}}}
         ]
-        unshelved = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
+        unshelve = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
 
-        return (expired, unshelved)
+        # get list of alerts to be unack'ed
+        pipeline = [
+            {'$match': {'status': 'ack'}},
+            {'$unwind': '$history'},
+            {'$match': {
+                'history.type': 'ack',
+                'history.status': 'ack'
+            }},
+            {'$sort': {'history.updateTime': -1}},
+            {'$group': {
+                '_id': '$_id',
+                'event': {'$first': '$event'},
+                'lastReceiveId': {'$first': '$lastReceiveId'},
+                'updateTime': {'$first': '$history.updateTime'},
+                'timeout': {'$first': '$timeout'}
+            }},
+            {'$project': {
+                'event': 1,
+                'lastReceiveId': 1,
+                'expireTime': {'$add': ['$updateTime', {'$multiply': ['$timeout', 1000]}]}
+            }},
+            {'$match': {'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {'$ne': 0}}}
+        ]
+        unack = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
+
+        return has_expired, unshelve + unack
