@@ -45,6 +45,7 @@ class NoCustomerMatch(AlertaException):
 
 class BaseError(Exception):
     code = 500
+    description = 'Unhandled exception'
 
     def __init__(self, message, code=None, errors=None):
         super().__init__(message)
@@ -75,6 +76,7 @@ class ExceptionHandlers:
 
 
 def handle_http_error(error: HTTPException) -> Tuple[Response, int]:
+    error.code = error.code or 500
     if error.code >= 500:
         current_app.logger.exception(error)
     return jsonify({
